#include <cstdlib>
#include <stdio.h>
#include <iostream>

#include <xercesc/dom/DOM.hpp>

#include "products/re/3.0/RapidEye_ProductMetadata_SensorLevel.hxx"

using namespace xercesc;

bool readReMetadata(std::string metadataFileName)
{
  XMLPlatformUtils::Initialize();
  
  try
  {
    std::auto_ptr<products::productMetadataSensor::earthObservationType> re = 
      products::productMetadataSensor::EarthObservation(metadataFileName,
      xml_schema::flags::dont_validate | xml_schema::flags::keep_dom | xml_schema::flags::dont_initialize);

    DOMNode* domNode = re->metaDataProperty().at(0)._node()->getFirstChild();
    //DEBUG: extract node name
    std::cout << xsd::cxx::xml::transcode<char>(domNode->getNodeName()) << std::endl;
    DOMElement* domElement = dynamic_cast<DOMElement*>(domNode);    

    products::productMetadataSensor::EarthObservationMetaDataType eoMetadata(*domElement, xml_schema::flags::dont_initialize);
    //DEBUG: check that everything is OK - try to get some value
    std::cout << eoMetadata.identifier().c_str() << std::endl;
  }
  catch (const xml_schema::exception& e)
  {
    std::cout << e << std::endl;
    return false;
  }
  
  XMLPlatformUtils::Terminate ();
  return true;
}

bool writeReMetadata(std::string metadataFileName)
{
  XMLPlatformUtils::Initialize();
  
  try
  {
    //boring stuff - creation and initialization of some metadata objects
    eop::EarthObservationMetaDataType::identifier_type identifier = 
      "2009-05-13T090541_RE3_1B-NAC_3436911_84312";

    eop::EarthObservationMetaDataType::acquisitionType_type acquisitionType(
      eop::EarthObservationMetaDataType::acquisitionType_type::NOMINAL);

    eop::EarthObservationMetaDataType::productType_type productType = "L1B";

    eop::EarthObservationMetaDataType::status_type statusType(
      eop::EarthObservationMetaDataType::status_type::ARCHIVED);

    products::productMetadataSensor::EarthObservationMetaDataType::license_type::licenseType1_type licType = 
      "Demonstration";

    const std::string resourceLinkValue = "RE EULA";
    products::productMetadataSensor::linkType::href_type resourceHref = 
      "http://info.rapideye.de/license/Demonstration_LicenseAgreement.txt";
    products::productMetadataSensor::EarthObservationMetaDataType::license_type::resourceLink_type resourceLink(
      resourceLinkValue, resourceHref);
    products::productMetadataSensor::EarthObservationMetaDataType::license_type licenseType(
      licType, resourceLink);

    products::productMetadataSensor::EarthObservationMetaDataType::orderId_type orderId = 84312;

    products::productMetadataSensor::EarthObservationMetaDataType::pixelFormat_type pixelFormat(
      products::productMetadataSensor::EarthObservationMetaDataType::pixelFormat_type::cxx_16U);

    //we are ready to create one of base objects - EarthObservationMetaDataType
    std::auto_ptr<products::productMetadataSensor::EarthObservationMetaDataType> eoMetadata(
      new products::productMetadataSensor::EarthObservationMetaDataType(
        identifier,
        acquisitionType,
        productType,
        statusType,
        licenseType,
        orderId,
        pixelFormat));

    //DEBUG - serialize object to stdout
    xml_schema::namespace_infomap infomap;
    const std::string encoding = "UTF-8";
    products::productMetadataSensor::EarthObservationMetaData(
      std::cout,
      *eoMetadata, 
      infomap,
      encoding,
      xml_schema::flags::dont_validate | xml_schema::flags::keep_dom |
    xml_schema::flags::dont_initialize | xml_schema::flags::no_xml_declaration);

    //create second base object  
    gml::AbstractGMLType::metaDataProperty_type metadataProperty;

    //The main problem - we need push first object to second
    ////////// This code block does not work correctly
    xml_schema::dom::auto_ptr<DOMDocument> eoMetadataDom = 
      products::productMetadataSensor::EarthObservationMetaData(
        *eoMetadata,
        infomap,
        xml_schema::flags::dont_validate | xml_schema::flags::keep_dom |
      xml_schema::flags::dont_initialize | xml_schema::flags::no_xml_declaration);

    eoMetadataDom->setUserData(xml_schema::dom::tree_node_key, eoMetadataDom.get(), 0);

    DOMElement* domElement = eoMetadataDom->getDocumentElement();
    //DEBUG
    std::cout << XMLString::transcode(domElement->getNodeName()) << std::endl;

    //set DOM to metadata type variable - an exception rises
    metadataProperty._node(domElement);
    ////////// End of code block

    gml::AbstractGMLType::metaDataProperty_sequence metadataSequence;
    metadataSequence.push_back(metadataProperty);

    //create main XML object
    products::productMetadataSensor::earthObservationType::validTime_type validTime;
    products::productMetadataSensor::earthObservationType::resultOf_type resultOf;
    products::productMetadataSensor::earthObservationType::re_standard_product_version_type productVersionType("VersionType");

    products::productMetadataSensor::earthObservationType eoType(
      validTime,
      resultOf,
      productVersionType);

    eoType.metaDataProperty(metadataSequence);

    //write resul to STDOUT file
    products::productMetadataSensor::EarthObservation(
      std::cout,
      eoType, 
      infomap,
      encoding,
      xml_schema::flags::dont_validate | xml_schema::flags::keep_dom |
    xml_schema::flags::dont_initialize);
  
  }
  catch (const xml_schema::exception& e)
  {
    std::cout << e << std::endl;
    return false;
  }
  
  XMLPlatformUtils::Terminate ();
  return true;
}


int main(int argc, char** argv) 
{
  //bool isOK = readReMetadata("/home/alexander/Downloads/RE_Sample/L1b/2009-05-13T090541_RE3_1B-NAC_3436911_84312_metadata.xml");
  bool isOK = writeReMetadata("/home/alexander/Temp/RE_metadata.xml");
  if (!isOK)
    std::cout << "ERROR occurred!" << std::endl;
  
  return 0;
}
